# -*- coding: utf-8 -*-

from plone.jsonapi.routes import add_plone_route as route

# CRUD
from plone.jsonapi.routes.api import get_record
from plone.jsonapi.routes.api import create_items
from plone.jsonapi.routes.api import update_items
from plone.jsonapi.routes.api import delete_items
from plone.jsonapi.routes.api import cut_items
from plone.jsonapi.routes.api import copy_items
from plone.jsonapi.routes.api import paste_items

from plone.jsonapi.routes.api import url_for


# GET
@route("/get", "get", methods=["GET"])
@route("/get/<string:uid>", "get", methods=["GET"])
def get(context, request, uid=None):
    """ get content
    """
    return get_record(uid=uid)


# CREATE
@route("/create", "create", methods=["POST"])
@route("/create/<string:uid>", "create", methods=["POST"])
def create(context, request, uid=None):
    """ create content
    """
    items = create_items(uid=uid, request=request)
    return {
        "url": url_for("create"),
        "count": len(items),
        "items": items,
    }


# UPDATE
@route("/update", "update", methods=["POST"])
@route("/update/<string:uid>", "update", methods=["POST"])
def update(context, request, uid=None):
    """ update content
    """
    items = update_items(uid=uid, request=request)
    return {
        "url": url_for("update"),
        "count": len(items),
        "items": items,
    }


# DELETE
@route("/delete", "delete", methods=["GET", "POST"])
@route("/delete/<string:uid>", "delete", methods=["GET", "POST"])
def delete(context, request, uid=None):
    """ delete content
    """
    items = delete_items(uid=uid, request=request)
    return {
        "url": url_for("delete"),
        "count": len(items),
        "items": items,
    }


# CUT
@route("/cut", "cut", methods=["GET", "POST"])
@route("/cut/<string:uid>", "cut", methods=["GET", "POST"])
def cut(context, request, uid=None):
    """ cut content
    """
    items = cut_items(uid=uid, request=request)
    return {
        "url": url_for("cut"),
        "count": len(items),
        "items": items,
    }


# COPY
@route("/copy", "copy", methods=["GET", "POST"])
@route("/copy/<string:uid>", "copy", methods=["GET", "POST"])
def copy(context, request, uid=None):
    """ copy content
    """
    items = copy_items(uid=uid, request=request)
    return {
        "url": url_for("copy"),
        "count": len(items),
        "items": items,
    }


# PASTE
@route("/paste", "paste", methods=["GET", "POST"])
@route("/paste/<string:uid>", "paste", methods=["GET", "POST"])
def paste(context, request, uid=None):
    """ paste content
    """
    items = paste_items(uid=uid, request=request)
    return {
        "url": url_for("paste"),
        "count": len(items),
        "items": items,
    }



# -*- coding: utf-8 -*-

import logging
import mimetypes
import datetime
import DateTime
import Missing

import simplejson as json

from zope import interface
from zope import component

from plone.dexterity.interfaces import IDexterityContent

from AccessControl import Unauthorized

from Products.CMFCore.interfaces import ISiteRoot
from Products.ZCatalog.interfaces import ICatalogBrain
from Products.ATContentTypes.interfaces import IATContentType

from plone.jsonapi.routes.interfaces import IInfo
from plone.jsonapi.routes.interfaces import IDataManager
from plone.jsonapi.routes.datamanagers import ATDataManager
from plone.jsonapi.routes.datamanagers import DexterityDataManager
from plone.jsonapi.routes import api
from plone.jsonapi.routes import request as req

import pkg_resources
try:
    pkg_resources.get_distribution('plone.app.textfield')
    from plone.app.textfield.interfaces import IRichTextValue
except (pkg_resources.DistributionNotFound, ImportError):
    HAS_PLONE_APP_TEXTFIELD = False
else:
    HAS_PLONE_APP_TEXTFIELD = True

__author__ = 'Ramon Bartl <ramon.bartl@googlemail.com>'
__docformat__ = 'plaintext'

_marker = object

logger = logging.getLogger("plone.jsonapi.routes")


class Base(object):
    """ Base Adapter
    """
    interface.implements(IInfo)

    def __init__(self, context):
        self.context = context
        self.keys = []
        self.ignore = []

        # Mapped attributes to extract from the object besides the schema keys.
        # These keys are always included
        self.attributes = {
            "id":          "getId",
            "uid":         "UID",
            "title":       "Title",
            "description": "Description",
            "created":     "created",
            "modified":    "modified",
            "effective":   "effective",
            "portal_type": "portal_type",
            "tags":        "Subject",
            "author":      "Creator",
        }

    def to_dict(self):
        """ extract the data of the content and return it as a dictionary
        """

        # 1. extract the schema fields
        data = extract_fields(self.context, self.keys, ignore=self.ignore)

        # 2. include custom key-value pairs listed in the mapping dictionary
        for key, attr in self.attributes.iteritems():
            # key already extracted in the first step
            if data.get(key, _marker) is not _marker:
                continue  # don't overwrite
            if key in self.ignore:
                continue  # skip ignores
            # fetch the mapped attribute
            value = getattr(self.context, attr, None)
            # handle function calls
            if callable(value):
                value = value()
            # map the value to the given key from the mapping
            data[key] = get_json_value(self.context, key, value)
        return data

    def __call__(self):
        return self.to_dict()


class ZCDataProvider(Base):
    """ Catalog Brain Adapter
    """
    interface.implements(IInfo)
    component.adapts(ICatalogBrain)

    def __init__(self, context):
        super(ZCDataProvider, self).__init__(context)
        catalog = api.get_portal_catalog()
        # extract the metadata
        self.keys = catalog.schema()

        # add specific catalog brain mappings
        self.attributes.update({
            "path": "getPath",
        })

        # ignore some metadata values, which we already mapped
        self.ignore = [
            'CreationDate',
            'Creator',
            'Date',
            'Description',
            'EffectiveDate',
            'ExpirationDate',
            'ModificationDate',
            'Subject',
            'Title',
            'Type',
            'UID',
            'cmf_uid',
            'getIcon',
            'getId',
            'getObjSize',
            'getRemoteUrl',
            'listCreators',
            'meta_type',
        ]


class DexterityDataProvider(Base):
    """ Data Provider for Dexterity based content types
    """
    interface.implements(IInfo)
    component.adapts(IDexterityContent)

    def __init__(self, context):
        super(DexterityDataProvider, self).__init__(context)

        # get the behavior and schema fields from the data manager
        dm = DexterityDataManager(context)
        schema = dm.get_schema()
        behaviors = dm.get_behaviors()

        self.keys = schema.names() + behaviors.keys()


class ATDataProvider(Base):
    """ Archetypes Adapter
    """
    interface.implements(IInfo)
    component.adapts(IATContentType)

    def __init__(self, context):
        super(ATDataProvider, self).__init__(context)

        # get the schema fields from the data manager
        dm = ATDataManager(context)
        schema = dm.get_schema()
        self.keys = schema.keys()


class SiteRootDataProvider(Base):
    """ Site Root Adapter
    """
    interface.implements(IInfo)
    component.adapts(ISiteRoot)

    def __init__(self, context):
        super(SiteRootDataProvider, self).__init__(context)
        # virtual keys, which are handled by the data manager
        self.keys = ["uid", "path"]


# ---------------------------------------------------------------------------
#   Functional Helpers
# ---------------------------------------------------------------------------

def extract_fields(obj, fieldnames, ignore=[]):
    """Extract the given fieldnames from the object

    :param obj: Content object
    :type obj: ATContentType/DexterityContentType
    :param ignore: Schema names to ignore
    :type ignore: list
    :returns: Schema name/value mapping
    :rtype: dict
    """

    # get the proper data manager for the object
    dm = IDataManager(obj)

    # filter out ignored fields
    fieldnames = filter(lambda name: name not in ignore, fieldnames)

    # schema mapping
    out = dict()

    for fieldname in fieldnames:
        try:
            # get the field value with the data manager
            fieldvalue = dm.get(fieldname)
        # https://github.com/collective/plone.jsonapi.routes/issues/52
        # -> skip restricted fields
        except Unauthorized:
            logger.debug("Skipping restricted field '%s'" % fieldname)
            continue

        out[fieldname] = get_json_value(obj, fieldname, fieldvalue)

    return out


def get_json_value(obj, fieldname, value=_marker, default=None):
    """JSON save value encoding

    :param obj: Content object
    :type obj: ATContentType/DexterityContentType
    :param fieldname: Schema name of the field
    :type fieldname: str/unicode
    :param value: The field value
    :type value: depends on the field type
    :returns: JSON encoded field value
    :rtype: field dependent
    """

    # check if the value is callable
    if callable(value):
        value = value()

    # returned from catalog brain metadata
    if value is Missing.Value:
        return default

    # extract the value from the object if omitted
    if value is _marker:
        value = IDataManager(obj).get(fieldname)

    # check if we have a date
    if is_date(value):
        return get_iso_date(value)

    # handle richtext values
    if is_richtext_value(value):
        value = value.output

    # check if the value is a file field
    if is_file_field(value):

        # Issue https://github.com/collective/plone.jsonapi.routes/issues/54
        # -> return file data only if requested
        if req.get_filedata(False):
            return get_file_info(obj, fieldname, value)

        # return the donwload url as the default file field value
        value = get_download_url(obj, fieldname, value)

    # check if the value is JSON serializable
    if not is_json_serializable(value):
        return default

    return value


def get_file_info(obj, fieldname, field=_marker, default=None):
    """Extract file data from a file field

    :param obj: Content object
    :type obj: ATContentType/DexterityContentType
    :param fieldname: Schema name of the field
    :type fieldname: str/unicode
    :param field: Blob field
    :type field: plone.app.blob.field.BlobWrapper
    :returns: File data mapping
    :rtype: dict
    """

    # extract the file field from the object if omitted
    if field is _marker:
        field = IDataManager(obj).get(fieldname)

    # check if we have a file field
    if not is_file_field(field):
        return default

    # extract file field attributes
    data = field.data.encode("base64")
    content_type = get_content_type(field)
    filename = getattr(field, "filename", "")
    download = get_download_url(obj, fieldname, field)

    return {
        "data": data,
        "size": len(field.data),
        "content_type": content_type,
        "filename": filename,
        "download": download,
    }


def get_download_url(obj, fieldname, field=_marker, default=None):
    """Calculate the download url

    :param obj: Content object
    :type obj: ATContentType/DexterityContentType
    :param fieldname: Schema name of the field
    :type fieldname: str/unicode
    :param field: The file field
    :type field: object
    :returns: The file download url
    :rtype: str
    """

    # extract the file field from the object if omitted
    if field is _marker:
        field = IDataManager(obj).get(fieldname)

    # check if we have a file field
    if not is_file_field(field):
        return default

    download = None
    if is_dexterity_content(obj):
        # calculate the download url
        filename = getattr(field, "filename", "")
        download = "{url}/@@download/{fieldname}/{filename}".format(
            url=obj.absolute_url(),
            fieldname=fieldname,
            filename=filename,
        )
    else:
        # calculate the download url
        download = "{url}/download".format(
            url=obj.absolute_url(),
        )
    return download


def get_content_type(obj, default="application/octet-stream"):
    """Get the content type of the file object

    :param obj: ATContentType, DexterityContentType, BlobWrapper
    :type obj: object
    :returns: The content type of the file
    :rtype: str
    """
    if hasattr(obj, "contentType"):
        if callable(obj.contentType):
            return obj.contentType()
        return obj.contentType
    elif hasattr(obj, "content_type"):
        if callable(obj.content_type):
            return obj.content_type()
        return obj.content_type
    filename = getattr(obj, "filename", "")
    content_type = mimetypes.guess_type(filename)[0]
    return content_type or default


def get_iso_date(date, default=None):
    """Get the ISO representation for the date object

    :param date: A date object
    :type field: datetime/DateTime
    :returns: The ISO format of the date
    :rtype: str
    """

    # not a date
    if not is_date(date):
        return default

    # handle Zope DateTime objects
    if isinstance(date, (DateTime.DateTime)):
        return date.ISO8601()

    # handle python datetime objects
    return date.isoformat()


def is_json_serializable(thing):
    """Checks if the given thing can be serialized to JSON

    :param thing: The object to check if it can be serialized
    :type thing: arbitrary object
    :returns: True if it can be JSON serialized
    :rtype: bool
    """
    try:
        json.dumps(thing)
        return True
    except TypeError:
        return False


def is_date(thing):
    """Checks if the given thing represents a date

    :param thing: The object to check if it is a date
    :type thing: arbitrary object
    :returns: True if we have a date object
    :rtype: bool
    """
    # known date types
    date_types = (datetime.datetime,
                  datetime.date,
                  DateTime.DateTime)
    return isinstance(thing, date_types)


def is_dexterity_content(obj):
    """Checks if the given object is Dexterity based

    :param obj: The content object to check
    :type thing: ATContentType/DexterityContentType
    :returns: True if the content object is Dexterity based
    :rtype: bool
    """
    return IDexterityContent.providedBy(obj)


def is_file_field(field):
    """Checks if the field is a file field

    :param field: The field to test
    :type thing: field object
    :returns: True if the field is a file field
    :rtype: bool
    """
    return hasattr(field, "filename")


def is_richtext_value(thing):
    """Checks if the value is a richtext value

    :param thing: The thing to test
    :type thing: any
    :returns: True if the thing is a richtext value
    :rtype: bool
    """
    if HAS_PLONE_APP_TEXTFIELD:
        return IRichTextValue.providedBy(thing)
    return False



# -*- coding: utf-8 -*-

import os
import re
import json
import datetime
import fileinput
import subprocess
import pkg_resources
from os.path import join as j

# Fabric imports
from fabric.api import env
from fabric.api import lcd
from fabric.api import task
from fabric.api import local
from fabric.colors import yellow
from fabric.colors import green
from fabric.tasks import execute

# -----------------------------------------------------------------------------
# CUSTOMER CUSTOM
# -----------------------------------------------------------------------------

REQUIREMENTS = "".split()
ENVIRONMENT = "HOME".split()
VERSION_PACKAGES = "plone.jsonapi.routes".split()


# -----------------------------------------------------------------------------
# FABRIC BOOTSTRAP
# -----------------------------------------------------------------------------

def cmd_exists(cmd):
    """ check if the command exists
    """
    return subprocess.call("type " + cmd,
                           shell=True,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE) == 0


# load the package distribution info into the environment
if not os.path.exists("package.json"):
    raise RuntimeError("No package.json file found in %s" % os.getcwd())


# check requirements
for req in REQUIREMENTS:
    if not cmd_exists(req):
        raise RuntimeError("Requirement `%s` not found in PATH" % req)


# check enviornment variables
for req in ENVIRONMENT:
    if req not in os.environ:
        raise RuntimeError("Environment Variable `%s` not set" % req)


# load the json config into the fabric environment
env.package = json.loads(file("package.json", "r").read())


# -----------------------------------------------------------------------------
# HELPER
# -----------------------------------------------------------------------------

def get_section(name="main"):
    """ returns the value of the section or an empty dict
    """
    return env.package.get(name, {})


def get_option(name, section="main", default=None):
    """ returns the value of the option
    """
    return get_section(section).get(name, default)


def get_customer():
    """ returns the customer name
    """
    return get_option("customer")


def get_packages():
    """ returns the packages
    """
    return get_option("packages", default=[])


def get_src_dir():
    """ returns the absolute path to the src directory
    """
    path = get_option("src_dir")
    return j(os.getcwd(), path)


def get_dist_dir():
    """ returns the absolute path to the dist directory
    """
    path = get_option("dist_dir")
    return j(os.getcwd(), path)


def get_docs_dir():
    """ returns the absolute path to the docs directory
    """
    path = get_option("docs_dir")
    return j(os.getcwd(), path)


def get_package_dir(package):
    """ returns the absolute path to the python package
    """
    src = get_src_dir()
    path = j(os.getcwd(), src, package)
    if os.path.exists(path):
        return path
    # different layout -- we are already inside the package
    return src


def get_distribution(package):
    """ get the pkg_resources distribution
    """
    try:
        return pkg_resources.get_distribution(package)
    except pkg_resources.DistributionNotFound:
        # setup pkg_resources to find the distribution
        pkg_resources.working_set.add_entry(get_package_dir(package))
        return pkg_resources.get_distribution(package)


def get_version(package):
    """ return the (stripped) version of the given package
    """
    dist = get_distribution(package)
    return dist.version.strip()


def get_version_file(package):
    """ return the version file of package or None
    """
    package_dir = get_package_dir(package)
    subdir = package.split(".")
    subdir.append("version.py")
    return j(package_dir, *subdir)


def write_version_info(package):
    """ updates the build and date of the version module

    The version.py must contain the version in the following format:
        __date__ = "2015-07-09"
        __build__ = 4711
        __version__ = 42
    """

    f = get_version_file(package)
    if not os.path.exists(f):
        raise RuntimeError("File '%s' does not exist." % f)

    for line in fileinput.input(f, inplace=1):
        if re.findall("__build__.*=", line):
            build = int(line.split("=")[1])
            line = "__build__ = %d" % (build + 1)
        elif re.findall("__date__.*=", line):
            now = datetime.datetime.now().strftime("%Y-%m-%d")
            line = "__date__ = '%s'" % now
        print line.strip("\n")


def get_full_version(package):
    """ returns the verion from the verion file
    """
    vf = get_version_file(package)
    out = {}
    lines = file(vf).readlines()
    for l in lines:
        if "=" in l and l.split("=")[0].strip() in \
           ("__build__", "__date__", "__version__"):
            name, value = l.split("=")
            name = name.strip(" _\n\r")
            value = value.strip(' "\'\n\r')
            # handle version method call gracefully
            out[name] = value.replace("version()", get_version(package))
    return out


# -----------------------------------------------------------------------------
# PUBLIC API
# -----------------------------------------------------------------------------

@task
def reload():
    execute(bump_version)
    local("wget --delete-after http://admin:admin@localhost:8080/@@reload?action=code")
    print green("RELOADED CODE")


@task
def make_docs():
    with lcd("docs"):
        local("make html")


@task
def preview_docs():
    with lcd("docs"):
        local("open _build/html/index.html")


@task
def bump_version():
    """ Bump up the version number
    """
    for package in VERSION_PACKAGES:
        version_file = get_version_file(package)
        print yellow("bumping version ...")
        write_version_info(package)
        print green("adding file '%s' to next commit." % version_file)
        local("git add " + version_file)
        print green(get_full_version(package))


@task
def versions():
    """ print the versions of the listed packages
    """
    for package in get_packages():
        dist = get_distribution(package)
        print green(
            "{} -> {}".format(
                package, dist.version.strip()))


@task
def build():
    """ Build all listed packages
    """
    # build all package
    for package in get_packages():
        print yellow("Building Package '%s'" % package)

        # build the package
        local("python setup.py sdist bdist_egg")
        print green("Finished Package '%s'" % package)



# -*- coding: utf-8 -*-

import logging

from zope import interface
from zope.schema import getFields
from zope.schema.interfaces import IObject

from plone import api
from plone.behavior.interfaces import IBehaviorAssignable

from AccessControl import Unauthorized
from AccessControl import getSecurityManager

from Products.CMFCore import permissions
from Products.Archetypes.utils import mapply

from plone.jsonapi.routes.interfaces import IDataManager
from plone.jsonapi.routes import underscore as _

import pkg_resources
try:
    pkg_resources.get_distribution('plone.app.textfield')
    from plone.app.textfield.interfaces import IRichText
    from plone.app.textfield.value import RichTextValue
except (pkg_resources.DistributionNotFound, ImportError):
    HAS_PLONE_APP_TEXTFIELD = False
else:
    HAS_PLONE_APP_TEXTFIELD = True

__author__ = 'Ramon Bartl <ramon.bartl@googlemail.com>'
__docformat__ = 'plaintext'

logger = logging.getLogger("plone.jsonapi.routes.datamanagers")


class BrainDataManager(object):
    """ Adapter to get catalog brain attributes
    """
    interface.implements(IDataManager)

    def __init__(self, context):
        self.context = context

    def get(self, name):
        """ get the value by name
        """
        # read the attribute
        attr = getattr(self.context, name, None)
        if callable(attr):
            return attr()
        return attr

    def set(self, name, value, **kw):
        """ Not used for catalog brains
        """
        logger.warn("set attributes not allowed on catalog brains")


class PortalDataManager(object):
    """ Adapter to set and get attributes of the Plone portal
    """
    interface.implements(IDataManager)

    def __init__(self, context):
        self.context = context

    def get(self, name):
        """ get the value by name
        """

        # check read permission
        sm = getSecurityManager()
        permission = permissions.View
        if not sm.checkPermission(permission, self.context):
            raise Unauthorized("Not allowed to view the Plone portal")

        # read the attribute
        attr = getattr(self.context, name, None)
        if callable(attr):
            return attr()

        # XXX no really nice, but we want the portal to behave like an ordinary
        # content type. Therefore we need to inject the neccessary data.
        if name == "uid":
            return 0
        if name == "path":
            return "/%s" % self.context.getId()
        return attr

    def set(self, name, value, **kw):
        """ Set the attribute to the given value.

        The keyword arguments represent the other attribute values
        to integrate constraints to other values.
        """

        # check write permission
        sm = getSecurityManager()
        permission = permissions.ManagePortal
        if not sm.checkPermission(permission, self.context):
            raise Unauthorized("Not allowed to modify the Plone portal")

        # set the attribute
        if not hasattr(self.context, name):
            return False
        self.context[name] = value
        return True


class ATDataManager(object):
    """ Adapter to set and get field values of AT Content Types
    """
    interface.implements(IDataManager)

    def __init__(self, context):
        self.context = context

    def get_schema(self):
        """ get the schema
        """
        return self.context.Schema()

    def is_file_field(self, field):
        """ checks if field is a file field
        """
        # XXX find a better way to distinguish file/image fields
        if getattr(field, "type", None) in ["file", "image", "blob"]:
            return True
        return False

    def get_field(self, name):
        """ return the field by name
        """
        return self.context.getField(name)

    def set(self, name, value, **kw):
        """ Set the field to the given value.

        The keyword arguments represent the other field values
        to integrate constraints to other values.
        """

        # fetch the field by name
        field = self.get_field(name)

        # bail out if we have no field
        if not field:
            return False

        # check the field permission
        if not field.checkPermission("write", self.context):
            raise Unauthorized("Not allowed to write the field %s" % name)

        # set the field value
        if self.is_file_field(field):
            logger.debug("ATDataManager::set:File field detected ('%r'), "
                         "base64 decoding value", field)
            value = str(value).decode("base64")
            # handle the filename
            if "filename" not in kw:
                logger.debug("ATDataManager::set: No Filename detected "
                             "-> using title or id")
                kw["filename"] = kw.get("id") or kw.get("title")

        # id fields take only strings
        if name == "id":
            value = str(value)

        # set the value to the field
        self._set(field, value, **kw)
        return True

    def _set(self, field, value, **kw):
        """ set the raw value of the field
        """
        logger.debug("ATDataManager::set: field=%r, value=%r", field, value)
        # get the field mutator
        mutator = field.getMutator(self.context)
        # Inspect function and apply positional and keyword arguments if
        # possible.
        return mapply(mutator, value, **kw)

    def get(self, name):
        """ get the value of the field by name
        """

        # fetch the field by name
        field = self.get_field(name)

        # bail out if we have no field
        if not field:
            return None

        # check the field permission
        if not field.checkPermission("read", self.context):
            raise Unauthorized("Not allowed to read the field %s" % name)

        # return the field value
        return field.get(self.context)


class DexterityDataManager(object):
    """ Adapter to set and get field values of Dexterity Content Types
    """
    interface.implements(IDataManager)

    def __init__(self, context):
        self.context = context
        self.schema = self.get_schema()
        self.behaviors = self.get_behaviors()

    def get_behaviors(self):
        """ Iterate over all behaviors that are assigned to the object
        Used code from @tisto:
        https://github.com/plone/plone.restapi/blob/master/src/plone/restapi/utils.py
        """
        out = {}
        assignable = IBehaviorAssignable(self.context, None)
        if not assignable:
            return out
        for behavior in assignable.enumerateBehaviors():
            for name, field in getFields(behavior.interface).items():
                out[name] = field
        return out

    def get_schema(self):
        pt = api.portal.get_tool("portal_types")
        fti = pt.getTypeInfo(self.context.portal_type)
        return fti.lookupSchema()

    def get_field(self, name):
        sf = self.schema.get(name)
        bf = self.behaviors.get(name)
        return sf or bf

    def is_file_field(self, field):
        """ checks if field is a file field
        """
        if _.is_string(field):
            field = self.get_field(field)
        if self.is_richtext_field(field):
            return False
        return IObject.providedBy(field)

    def is_richtext_field(self, field):
        """ checks if field is a rich-text field
        """
        if _.is_string(field):
            field = self.get_field(field)
        if HAS_PLONE_APP_TEXTFIELD:
            return IRichText.providedBy(field)
        return False

    def get_filename(self, **kw):
        """ extract the filename from the keywords
        """
        if "filename" not in kw:
            logger.debug("ATDataManager::get_filename:No Filename detected"
                         "-- using title or id")
            kw["filename"] = kw.get("id") or kw.get("title")
        return kw.get("filename")

    def get_content_type(self, **kw):
        """ extract the mimetype from the keywords
        """
        if "mimetype" in kw:
            return kw.get("mimetype")
        if "content_type" in kw:
            # same key as in JSON response
            return kw.get("content_type")
        return None

    def can_write(self, field):
        """ check if the field is writeable
        """
        # check if the field is read only
        if field.readonly:
            raise Unauthorized("Field '%s' is read-only" % field.__name__)

        # XXX: How to check security on the field level?
        sm = getSecurityManager()
        permission = permissions.ModifyPortalContent
        if not sm.checkPermission(permission, self.context):
            raise Unauthorized("Not allowed to modify this content")
        return field

    def can_read(self, field):
        """ check if the field is readable
        """
        # XXX: How to check security on the field level?
        sm = getSecurityManager()
        if not sm.checkPermission(permissions.View, self.context):
            raise Unauthorized("Not allowed to view this content")
        return field

    def set(self, name, value, **kw):
        """ Set the field to the given value.

        The keyword arguments represent the other field values
        to integrate constraints to other values.
        """
        field = self.get_field(name)

        # bail out if we have no field
        if not field:
            return False

        # Check the write permission of the field
        self.can_write(field)

        if self.is_richtext_field(field):
            logger.debug("DexterityDataManager::set:RichText field"
                         "detected ('%r'), creating RichTextValue", field)
            value = RichTextValue(raw=value,
                                  outputMimeType=field.output_mime_type)

        if self.is_file_field(field):
            logger.debug("DexterityDataManager::set:File field"
                         "detected ('%r'), base64 decoding value", field)
            data = str(value).decode("base64")
            filename = self.get_filename(**kw)
            contentType = self.get_content_type(**kw)

            if contentType:
                # create NamedFile with content type information
                value = field._type(data=data, contentType=contentType,
                                    filename=filename)
            else:
                # create NamedFile w/o content type information
                # -> will be guessed by the extension of the filename
                value = field._type(data=data, filename=filename)

        # only possible with non file fields
        if not self.is_file_field(field):
            # validate the field value
            field.validate(value)

        logger.debug("DexterityDataManager::set:"
                     "field=%r, value=%r", field, value)

        field.set(self.context, value)
        return True

    def get(self, name):
        """ get the value of the field by name
        """
        field = self.get_field(name)
        if field is None:
            return None
        # Check the read permission of the field
        self.can_read(field)
        return field.get(self.context)



# -*- coding: utf-8 -*-

import os

from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles

from plone.jsonapi.routes.tests.base import APITestCase
from plone.jsonapi.routes import api
from plone.jsonapi.routes import adapters
from plone.jsonapi.routes.interfaces import IInfo
from plone.jsonapi.routes.interfaces import IDataManager


FILENAME = u"TestDoc.docx"


def dummy_file():
    from plone.namedfile.file import NamedBlobImage
    path = os.path.join(os.path.dirname(__file__), FILENAME)
    return NamedBlobImage(
        data=open(path, 'r').read(),
        filename=FILENAME
    )


class TestAdapters(APITestCase):
    """ Test the Data Adapter
    """

    def setUp(self):
        self.portal = self.getPortal()
        self.request = self.getRequest()

        # the test folder we created
        self.test_folder = self.portal.folder

        # give the test user the manager role
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        # Add a Word Document
        path = os.path.join(os.path.dirname(__file__), u'TestDoc.docx')
        file_contents = open(path).read()
        _ = self.portal.invokeFactory("File",
                                      "testdoc.docx",
                                      title=FILENAME,
                                      file=file_contents)
        self.doc = self.portal.get(_)

        # handle plone 5 dexterity based file content
        if adapters.is_dexterity_content(self.doc):
            self.doc.file = dummy_file()
        else:
            self.doc.setFilename(FILENAME)

    # -----------------------------------------------------------------------------
    #   Testing Helpers
    # -----------------------------------------------------------------------------

    def get_document_obj(self):
        _, obj = self.test_folder.items()[0]
        return obj

    def get_document_brain(self):
        pc = api.get_portal_catalog()
        results = pc({"portal_type": "Document", "limit": 1})
        return results[0]

    # -----------------------------------------------------------------------------
    #   Test Adapters
    # -----------------------------------------------------------------------------

    def test_brain_adapter(self):
        brain = self.get_document_brain()
        adapter = IInfo(brain)
        data = adapter()
        self.assertEqual(data.get("uid"), brain.UID)

    def test_at_adapter(self):
        obj = self.get_document_obj()
        adapter = IInfo(obj)
        data = adapter()
        self.assertEqual(data.get("uid"), obj.UID())

    def test_portal_adapter(self):
        obj = self.portal
        adapter = IInfo(obj)
        data = adapter()
        self.assertEqual(data.get("uid"), 0)

    # -----------------------------------------------------------------------------
    #   Test Functional Helpers
    # -----------------------------------------------------------------------------

    def test_extract_fields(self):
        # extract fields from content objects
        obj = self.get_document_obj()
        data = adapters.extract_fields(obj, ["title"])
        self.assertEqual(data.get("title"), obj.title)
        # extract fields from catalog brains
        brain = self.get_document_brain()
        data = adapters.extract_fields(brain, ["UID"])
        self.assertEqual(data.get("UID"), brain.UID)

    def test_get_json_value(self):
        # Ensure JSON serializable handling of a Schema field value
        obj = self.get_document_obj()

        # handles date objects
        # value = adapters.get_json_value(obj, "creation_date")
        # self.assertEqual(value, obj.created().ISO8601())

        # extracts the value by name if omitted
        value = adapters.get_json_value(obj, "title")
        self.assertEqual(value, obj.Title())

        # takes the given value
        value = adapters.get_json_value(obj, "foo", 1)
        self.assertEqual(type(value), int)

        # returns the default if the value is not JSON serializable
        value = adapters.get_json_value(obj, "object", object(),
                                        default="Not JSON serializable")
        self.assertEqual(value, "Not JSON serializable")

    def test_get_file_info(self):
        obj = self.doc
        # extract the file info
        info = adapters.get_file_info(obj, "file")
        self.assertEqual(info.get("filename"), FILENAME)

    def test_get_download_url(self):
        obj = self.doc
        # extract the file info
        info = adapters.get_download_url(obj, "file")
        self.assertTrue(info.find("download") > -1)

    def test_get_content_type(self):
        obj = self.doc
        # extracts the content type of content objects
        ct = adapters.get_content_type(obj)
        self.assertNotEquals(ct, "binary")
        # it handles non known objects gracefully
        ct = adapters.get_content_type(object(), "binary")
        self.assertEqual(ct, "binary")

    def test_get_iso_date(self):
        obj = self.get_document_obj()
        created = obj.created()
        date = adapters.get_iso_date(created)
        self.assertEqual(date, obj.created().ISO8601())

    def test_is_dexterity_content(self):
        obj = self.get_document_obj()
        if api.is_plone5():
            # std. content types are dexterity in plone 5
            self.assertTrue(adapters.is_dexterity_content(obj))
        else:
            self.assertFalse(adapters.is_dexterity_content(obj))

    def test_is_file_field(self):
        obj = self.doc
        dm = IDataManager(obj)
        field = dm.get_field("file")
        self.assertTrue(dm.is_file_field(field))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAdapters))
    return suite



# -*- coding: utf-8 -*-

from plone.jsonapi.routes import add_plone_route as route

# CRUD
from plone.jsonapi.routes.api import get_batched
from plone.jsonapi.routes.api import create_items
from plone.jsonapi.routes.api import update_items
from plone.jsonapi.routes.api import delete_items

from plone.jsonapi.routes.api import url_for


# GET
@route("/events", "events", methods=["GET"])
@route("/events/<string:uid>", "events", methods=["GET"])
def get(context, request, uid=None):
    """ get events
    """
    return get_batched("Event", uid=uid, endpoint="events")


# CREATE
@route("/events/create", "events_create", methods=["POST"])
@route("/events/create/<string:uid>", "events_create", methods=["POST"])
def create(context, request, uid=None):
    """ create events
    """
    items = create_items("Event", uid=uid, endpoint="events")
    return {
        "url": url_for("events_create"),
        "count": len(items),
        "items": items,
    }


# UPDATE
@route("/events/update", "events_update", methods=["POST"])
@route("/events/update/<string:uid>", "events_update", methods=["POST"])
def update(context, request, uid=None):
    """ update events
    """
    items = update_items("Event", uid=uid, endpoint="events")
    return {
        "url": url_for("events_update"),
        "count": len(items),
        "items": items,
    }


# DELETE
@route("/events/delete", "events_delete", methods=["POST"])
@route("/events/delete/<string:uid>", "events_delete", methods=["POST"])
def delete(context, request, uid=None):
    """ delete events
    """
    items = delete_items("Event", uid=uid, endpoint="events")
    return {
        "url": url_for("events_delete"),
        "count": len(items),
        "items": items,
    }



# -*- coding: utf-8 -*-

import logging
from zope.interface import Interface
from zope.schema import Bool

logger = logging.getLogger("plone.jsonapi.routes.zcml")


class IConfig(Interface):
    """ Plone JSONAPI Config Interface
    """

    use_advanced_query = Bool(
        title=u"Search uses Advanced Query",
        description=u"""
        Use Advanced Query if installed for catalog searches
        """,
        required=False)


def configDirective(_context, use_advanced_query=False):

    if use_advanced_query:
        from plone.jsonapi.routes import query
        logger.info("*** USE ADVANCED QUERY ***")
        query.USE_ADVANCED_QUERY = use_advanced_query
